@@ -79,13 +79,17 @@ BEGIN {
 
     for my $func ( @EXPORT ) {
         no strict 'refs';
-        
+
         my $prefix  = 'cp_';
         my $name    = $func;
         $name       =~ s/^$prefix//g;
-        
+
         *$func = sub {
                         my $msg     = shift;
+
+                        ### no point storing non-messages
+                        return unless defined $msg;
+
                         $log->store(
                                 message => $msg,
                                 tag     => uc $name,
@@ -96,7 +100,9 @@ BEGIN {
     }
 
     sub flush {
-        return reverse $log->flush;
+        my @foo = $log->flush;
+        return unless @foo;
+        return reverse @foo;
     }
 
     sub stack {
@@ -129,12 +135,16 @@ printed. This defaults to C<*STDERR>.
 This is the filehandle all the messages sent to C<msg()> are being
 printed. This default to C<*STDOUT>.
 
+=back
+
 =cut
+
 local $| = 1;
 $ERROR_FH   = \*STDERR;
 $MSG_FH     = \*STDOUT;
 
-package Log::Message::Handlers;
+package # Hide from Pause
+  Log::Message::Handlers;
 use Carp ();
 
 {
@@ -164,7 +174,10 @@ use Carp ();
         my $old_fh = select $CPANPLUS::Error::ERROR_FH;
 
         ### is only going to be 1 for now anyway ###
-        my $cb      = (CPANPLUS::Internals->_return_all_objects)[0];
+        ### C::I may not be loaded, so do a can() check first
+        my $cb      = CPANPLUS::Internals->can('_return_all_objects')
+                        ? (CPANPLUS::Internals->_return_all_objects)[0]
+                        : undef;
 
         ### maybe we didn't initialize an internals object (yet) ###
         my $debug   = $cb ? $cb->configure_object->get_conf('debug') : 0;
